/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.mobile.util;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.openide.util.Lookup;
import org.openide.util.LookupListener;
import it.tidalwave.util.logging.Logger;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class LazyLookup extends Lookup
  {
    private static final String CLASS = LazyLookup.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private final Map<Class<?>, Class<?>> classMap = new HashMap<Class<?>, Class<?>>();
    private final Map<Class<?>, Object> objectMap = new HashMap<Class<?>, Object>();

    public void register (final @Nonnull Class<?> interfaceClass, final @Nonnull Class<?> implementationClass)
      {
        classMap.put(interfaceClass, implementationClass);
      }

    @Override @CheckForNull
    public synchronized <T> T lookup (final @Nonnull Class<T> clazz)
      {
        logger.fine("lookup(%s)", clazz);
        final Collection<? extends T> instances = lookup(new Template<T>(clazz)).allInstances();
        return instances.isEmpty() ? null : instances.iterator().next();
      }

    @Override @CheckForNull
    public <T> Result<T> lookup (final @Nonnull Template<T> template)
      {
        logger.fine("lookup(%s)", template);
        final Class<? extends T> clazz = template.getType();

        return new Result<T>()
          {
            @Override
            public void addLookupListener(LookupListener l)
              {
              }

            @Override
            public void removeLookupListener(LookupListener l)
              {
              }

            @Override @Nonnull
            public Collection<? extends T> allInstances()
              {
                final List<T> results = new ArrayList<T>();
                T instance = (T)objectMap.get(clazz);

                if (instance == null)
                  {
                    final Class<? extends T> implementationClass = (Class<? extends T>)classMap.get(clazz);

                    if (implementationClass != null)
                      {
                        try
                          {
                            instance = implementationClass.newInstance();
                          }
                        catch (InstantiationException e)
                          {
                            throw new RuntimeException(e);
                          }
                        catch (IllegalAccessException e)
                          {
                            throw new RuntimeException(e);
                          }

                        objectMap.put(clazz, instance);
                      }
                  }

                if (instance != null)
                  {
                    results.add(instance);
                  }

//                logger.finest(">>>> returning %s", results);
                return results;
              }
          };
      }
  }
